/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.wtp.releng.tools.component.adopters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import org.eclipse.wtp.releng.tools.component.ILocation;
import org.eclipse.wtp.releng.tools.component.api.ComponentXMLVisitor;
import org.eclipse.wtp.releng.tools.component.internal.Location;
import org.eclipse.wtp.releng.tools.component.model.ComponentXML;
import org.eclipse.wtp.releng.tools.component.model.Package;
import org.eclipse.wtp.releng.tools.component.model.Plugin;
import org.eclipse.wtp.releng.tools.component.model.Type;
import org.eclipse.wtp.releng.tools.component.util.CommandOptionParser;
import org.eclipse.wtp.releng.tools.component.xsl.XSLUtil;

public class CombineClass2Reference
{
  private HashMap plugin2complead;
  ComponentLead chuck;
  ComponentLead david;
  ComponentLead derping;
  ComponentLead chris;
  ComponentLead craig;
  ComponentLead tim;
  ComponentLead unknown;
  private Collection src;
  private String output;
  private Collection api;

  private class ComponentLead
  {
    public String leadName;
    public TreeMap class2refCount;
    public TreeMap pkg2refCount;
    public TreeMap plugin2refCount;

    public ComponentLead(String leadName)
    {
      this.leadName = leadName;
      class2refCount = new TreeMap();
      pkg2refCount = new TreeMap();
      plugin2refCount = new TreeMap();
    }
  }

  private class UsageCount
  {
    public int apiUse = 0;
    public int nonAPIUse = 0;
  }

  public CombineClass2Reference()
  {
    chuck = new ComponentLead("Chuck");
    david = new ComponentLead("David");
    derping = new ComponentLead("Der Ping");
    chris = new ComponentLead("Chris");
    craig = new ComponentLead("Craig");
    tim = new ComponentLead("Tim");
    unknown = new ComponentLead("Unknown");
    plugin2complead = new HashMap();
    plugin2complead.put("org.eclipse..st.common.(?!env|snippet|uri).*", chuck);
    plugin2complead.put("org.eclipse..st.common.env.*", chris);
    plugin2complead.put("org.eclipse..st.common.snippet.*", david);
    plugin2complead.put("org.eclipse..st.common.uri.*", craig);
    plugin2complead.put("org.eclipse..st.j2ee.*", chuck);
    plugin2complead.put("org.eclipse..st.ejb.*", chuck);
    plugin2complead.put("org.eclipse..st.jsp.*", david);
    plugin2complead.put("org.eclipse..st.server.*", tim);
    plugin2complead.put("org.eclipse..st.servlet.*", chuck);
    plugin2complead.put("org.eclipse..st.ws", chris);
    plugin2complead.put("org.eclipse..st.ws[.].*", chris);
    plugin2complead.put("org.eclipse..st.command.*", chris);
    plugin2complead.put("org.eclipse..st.css.*", david);
    plugin2complead.put("org.eclipse..st.dtd.*", david);
    plugin2complead.put("org.eclipse..st.html.*", david);
    plugin2complead.put("org.eclipse..st.javascript.*", david);
    plugin2complead.put("org.eclipse..st.rdb.*", derping);
    plugin2complead.put("org.eclipse..st.sse.*", david);
    plugin2complead.put("org.eclipse..st.validation.*", chuck);
    plugin2complead.put("org.eclipse..st.web.*", chuck);
    plugin2complead.put("org.eclipse..st.wsdl.*", craig);
    plugin2complead.put("org.eclipse..st.xml.*", david);
    plugin2complead.put("org.eclipse..st.xsd.*", craig);
    plugin2complead.put("org.eclipse..st.internet.*", tim);
  }

  public Collection getSrc()
  {
    return src;
  }

  public void setSrc(Collection src)
  {
    this.src = src;
  }

  public String getOutput()
  {
    return output;
  }

  public void setOutput(String output)
  {
    this.output = output;
  }

  public Collection getApi()
  {
    return api;
  }

  public void setApi(Collection api)
  {
    this.api = api;
  }

  public void execute()
  {
    // Collect component.xml files
    Map pluginId2CompXML = new HashMap();
    if (api != null)
    {
      for (Iterator i = api.iterator(); i.hasNext();)
      {
        ILocation apiLocation = Location.createLocation(new File((String)i.next()));
        ComponentXMLVisitor compXMLVisitor = new ComponentXMLVisitor();
        apiLocation.accept(compXMLVisitor);
        for (Iterator it = compXMLVisitor.getCompXMLs().iterator(); it.hasNext();)
        {
          ComponentXML compXML = (ComponentXML)it.next();
          for (Iterator it2 = compXML.getPlugins().iterator(); it2.hasNext();)
          {
            pluginId2CompXML.put(((Plugin)it2.next()).getId(), compXML);
          }
        }
      }
    }
    for (Iterator it = src.iterator(); it.hasNext();)
    {
      FileInputStream fis = null;
      try
      {
        String file = (String)it.next();
        fis = new FileInputStream(file);
        References refs = new References();
        refs.load(fis);
        for (Iterator it2 = refs.getPluginRefs().iterator(); it2.hasNext();)
        {
          PluginRef pluginRef = (PluginRef)it2.next();
          String pluginId = pluginRef.getId();
          ComponentXML compXML = (ComponentXML)pluginId2CompXML.get(pluginId);
          ComponentLead compLead = unknown;
          for (Iterator it3 = plugin2complead.keySet().iterator(); it3.hasNext();)
          {
            String regex = (String)it3.next();
            if (pluginId.matches(regex))
            {
              compLead = (ComponentLead)plugin2complead.get(regex);
              break;
            }
          }
          for (Iterator it3 = pluginRef.getClassRefs().iterator(); it3.hasNext();)
          {
            ClassRef classRef = (ClassRef)it3.next();
            String name = classRef.getName();
            String pkgName = getPackageName(name);
            UsageCount usageCount = getUsageCount(compXML, classRef, pkgName, name.substring(pkgName.length() + 1));
            UsageCount classUsageCount = (UsageCount)compLead.class2refCount.get(name);
            if (classUsageCount == null)
            {
              classUsageCount = new UsageCount();
            }
            classUsageCount.apiUse += usageCount.apiUse;
            classUsageCount.nonAPIUse += usageCount.nonAPIUse;
            compLead.class2refCount.put(name, classUsageCount);
            UsageCount pkgUsageCount = (UsageCount)compLead.pkg2refCount.get(pkgName);
            if (pkgUsageCount == null)
            {
              pkgUsageCount = new UsageCount();
            }
            pkgUsageCount.apiUse += usageCount.apiUse;
            pkgUsageCount.nonAPIUse += usageCount.nonAPIUse;
            compLead.pkg2refCount.put(pkgName, pkgUsageCount);
            UsageCount pluginUsageCount = (UsageCount)compLead.plugin2refCount.get(pluginId);
            if (pluginUsageCount == null)
            {
              pluginUsageCount = new UsageCount();
            }
            pluginUsageCount.apiUse += usageCount.apiUse;
            pluginUsageCount.nonAPIUse += usageCount.nonAPIUse;
            compLead.plugin2refCount.put(pluginId, pluginUsageCount);
          }
        }
      }
      catch (Throwable t)
      {
        throw new RuntimeException(t);
      }
      finally
      {
        if (fis != null)
        {
          try
          {
            fis.close();
          }
          catch (IOException ioe)
          {
          }
        }
      }
    }
    genCSV();
    genHTML();
  }

  private UsageCount getUsageCount(ComponentXML compXML, ClassRef classRef, String pkgName, String localName)
  {
    UsageCount usageCount = new UsageCount();
    int refCount = classRef.getRefCount();
    int subclassCount = classRef.getSubclassCount();
    int implCount = classRef.getImplementCount();
    int instantiateCount = classRef.getInstantiateCount();
    if (compXML == null)
    {
      usageCount.nonAPIUse = refCount + subclassCount + implCount + instantiateCount;
      return usageCount;
    }
    Package pkg = compXML.getPackage(pkgName);
    if (pkg == null)
    {
      usageCount.nonAPIUse = refCount + subclassCount + implCount + instantiateCount;
      return usageCount;
    }
    else
    {
      Type type = pkg.getType(localName);
      if (type == null)
      {
        if (pkg.isApi())
        {
          usageCount.apiUse = refCount + subclassCount + implCount + instantiateCount;
          return usageCount;
        }
        else
        {
          usageCount.nonAPIUse = refCount + subclassCount + implCount + instantiateCount;
          return usageCount;
        }
      }
      else
      {
        if (!type.isReference())
        {
          usageCount.nonAPIUse += classRef.getRefCount();
        }
        else
        {
          usageCount.apiUse += classRef.getRefCount();
        }
        if (!type.isSubclass())
        {
          usageCount.nonAPIUse += classRef.getSubclassCount();
        }
        else
        {
          usageCount.apiUse += classRef.getSubclassCount();
        }
        if (!type.isImplement())
        {
          usageCount.nonAPIUse += classRef.getImplementCount();
        }
        else
        {
          usageCount.apiUse += classRef.getImplementCount();
        }
        if (!type.isInstantiate())
        {
          usageCount.nonAPIUse += classRef.getInstantiateCount();
        }
        else
        {
          usageCount.apiUse += classRef.getInstantiateCount();
        }
        return usageCount;
      }
    }
  }

  private String getPackageName(String className)
  {
    int i = className.lastIndexOf('.');
    if (i != -1)
    {
      return className.substring(0, i);
    }
    else
    {
      return "";
    }
  }

  public void genCSV()
  {
    FileWriter classWriter = null;
    FileWriter pkgWriter = null;
    FileWriter pluginWriter = null;
    try
    {
      classWriter = new FileWriter(output + ".class.csv");
      pkgWriter = new FileWriter(output + ".pkg.csv");
      pluginWriter = new FileWriter(output + ".plugin.csv");
      writeCompLeadCSV(chuck, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(david, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(derping, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(chris, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(craig, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(tim, classWriter, pkgWriter, pluginWriter);
      writeCompLeadCSV(unknown, classWriter, pkgWriter, pluginWriter);
    }
    catch (IOException ioe)
    {
      throw new RuntimeException(ioe);
    }
    finally
    {
      if (classWriter != null)
      {
        try
        {
          classWriter.close();
        }
        catch (IOException ioe)
        {
        }
      }
      if (pkgWriter != null)
      {
        try
        {
          pkgWriter.close();
        }
        catch (IOException ioe)
        {
        }
      }
      if (pluginWriter != null)
      {
        try
        {
          pluginWriter.close();
        }
        catch (IOException ioe)
        {
        }
      }
    }
  }

  private void writeCompLeadCSV(ComponentLead compLead, Writer classWriter, Writer pkgWriter, Writer pluginWriter) throws IOException
  {
    classWriter.write(compLead.leadName);
    classWriter.write("\n");
    for (Iterator it = compLead.class2refCount.keySet().iterator(); it.hasNext();)
    {
      String name = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.class2refCount.get(name);
      classWriter.write(name);
      classWriter.write(",");
      classWriter.write(String.valueOf(usageCount.nonAPIUse));
      classWriter.write(",");
      classWriter.write(String.valueOf(usageCount.apiUse));
      classWriter.write("\n");
    }
    classWriter.write("\n");

    pkgWriter.write(compLead.leadName);
    pkgWriter.write("\n");
    for (Iterator it = compLead.pkg2refCount.keySet().iterator(); it.hasNext();)
    {
      String name = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.pkg2refCount.get(name);
      pkgWriter.write(name);
      pkgWriter.write(",");
      pkgWriter.write(String.valueOf(usageCount.nonAPIUse));
      pkgWriter.write(",");
      pkgWriter.write(String.valueOf(usageCount.apiUse));
      pkgWriter.write("\n");
    }
    pkgWriter.write("\n");
    pluginWriter.write(compLead.leadName);
    pluginWriter.write("\n");
    for (Iterator it = compLead.plugin2refCount.keySet().iterator(); it.hasNext();)
    {
      String id = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.plugin2refCount.get(id);
      pluginWriter.write(id);
      pluginWriter.write(",");
      pluginWriter.write(String.valueOf(usageCount.nonAPIUse));
      pluginWriter.write(",");
      pluginWriter.write(String.valueOf(usageCount.apiUse));
      pluginWriter.write("\n");
    }
    pluginWriter.write("\n");
  }

  public void genHTML()
  {
    try
    {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      baos.write("<root>".getBytes());
      writeCompLead(chuck, baos);
      writeCompLead(david, baos);
      writeCompLead(derping, baos);
      writeCompLead(chris, baos);
      writeCompLead(craig, baos);
      writeCompLead(tim, baos);
      writeCompLead(unknown, baos);
      baos.write("</root>".getBytes());
      baos.close();
      byte[] content = baos.toByteArray();
      XSLUtil.transform
      (
        ClassLoader.getSystemResourceAsStream("org/eclipse/wtp/releng/tools/component/xsl/combine-plugin2ref.xsl"),
        new ByteArrayInputStream(content),
        new FileOutputStream(output + ".plugin.html")
      );
      XSLUtil.transform
      (
        ClassLoader.getSystemResourceAsStream("org/eclipse/wtp/releng/tools/component/xsl/combine-pkg2ref.xsl"),
        new ByteArrayInputStream(content),
        new FileOutputStream(output + ".pkg.html")
      );
      XSLUtil.transform
      (
        ClassLoader.getSystemResourceAsStream("org/eclipse/wtp/releng/tools/component/xsl/combine-class2ref.xsl"),
        new ByteArrayInputStream(content),
        new FileOutputStream(output + ".class.html")
      );
    }
    catch (Throwable t)
    {
      t.printStackTrace();
      throw new RuntimeException(t);
    }
  }

  private void writeCompLead(ComponentLead compLead, ByteArrayOutputStream baos) throws IOException
  {
    baos.write("<team lead=\"".getBytes());
    baos.write(compLead.leadName.getBytes());
    baos.write("\">".getBytes());
    for (Iterator it = compLead.plugin2refCount.keySet().iterator(); it.hasNext();)
    {
      String id = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.plugin2refCount.get(id);
      baos.write("<plugin id=\"".getBytes());
      baos.write(id.getBytes());
      baos.write("\" api=\"".getBytes());
      baos.write(String.valueOf(usageCount.apiUse).getBytes());
      baos.write("\" internal=\"".getBytes());
      baos.write(String.valueOf(usageCount.nonAPIUse).getBytes());
      baos.write("\"/>".getBytes());
    }
    for (Iterator it = compLead.pkg2refCount.keySet().iterator(); it.hasNext();)
    {
      String name = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.pkg2refCount.get(name);
      baos.write("<package name=\"".getBytes());
      baos.write(name.getBytes());
      baos.write("\" api=\"".getBytes());
      baos.write(String.valueOf(usageCount.apiUse).getBytes());
      baos.write("\" internal=\"".getBytes());
      baos.write(String.valueOf(usageCount.nonAPIUse).getBytes());
      baos.write("\"/>".getBytes());
    }
    for (Iterator it = compLead.class2refCount.keySet().iterator(); it.hasNext();)
    {
      String name = (String)it.next();
      UsageCount usageCount = (UsageCount)compLead.class2refCount.get(name);
      baos.write("<class name=\"".getBytes());
      baos.write(name.getBytes());
      baos.write("\" api=\"".getBytes());
      baos.write(String.valueOf(usageCount.apiUse).getBytes());
      baos.write("\" internal=\"".getBytes());
      baos.write(String.valueOf(usageCount.nonAPIUse).getBytes());
      baos.write("\"/>".getBytes());
    }
    baos.write("</team>".getBytes());
  }

  public static void main(String[] args)
  {
    CommandOptionParser optionParser = new CommandOptionParser(args);
    Map options = optionParser.getOptions();
    Collection src = (Collection)options.get("src");
    Collection output = (Collection)options.get("output");
    Collection api = (Collection)options.get("api");
    if (src == null || output == null || src.isEmpty() || output.isEmpty())
    {
      printUsage();
      System.exit(-1);
    }

    CombineClass2Reference class2Ref = new CombineClass2Reference();
    class2Ref.setSrc(src);
    class2Ref.setOutput((String)output.iterator().next());
    class2Ref.setApi(api);
    class2Ref.execute();
  }

  private static void printUsage()
  {
    System.out.println("Usage: java org.eclipse.wtp.releng.tools.component.adopters.CombineClass2Reference -src <src> -output <output>");
    System.out.println("");
    System.out.println("\t-src\t\t<src>\t\tlocation of your usage reports");
    System.out.println("\t-output\t<output>\t\tlocation of the output file");
    System.out.println("\t-api\t\t<api>\t\tlocation of your component.xml");
  }
}