/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.wtp.releng.tools.component.adopters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.wtp.releng.tools.component.util.CommandOptionParser;
import org.eclipse.wtp.releng.tools.component.xsl.XSLUtil;

public class ExtensionPointScanner implements IPlatformRunnable
{
  private Collection filters;
  private String output;
  private Collection includes;
  private Collection excludes;

  private HashMap plugin2complead;
  ComponentLead chuck;
  ComponentLead david;
  ComponentLead derping;
  ComponentLead chris;
  ComponentLead craig;
  ComponentLead tim;
  ComponentLead unknown;

  private class ComponentLead
  {
    public String leadName;
    public TreeMap extpt2RefCount;

    public ComponentLead(String leadName)
    {
      this.leadName = leadName;
      extpt2RefCount = new TreeMap();
    }
  }

  public ExtensionPointScanner()
  {
    chuck = new ComponentLead("Chuck");
    david = new ComponentLead("David");
    derping = new ComponentLead("Der Ping");
    chris = new ComponentLead("Chris");
    craig = new ComponentLead("Craig");
    tim = new ComponentLead("Tim");
    unknown = new ComponentLead("Unknown");
    plugin2complead = new HashMap();
    plugin2complead.put("org.eclipse..st.common.(?!env|snippet|uri).*", chuck);
    plugin2complead.put("org.eclipse..st.common.env.*", chris);
    plugin2complead.put("org.eclipse..st.common.snippet.*", david);
    plugin2complead.put("org.eclipse..st.common.uri.*", craig);
    plugin2complead.put("org.eclipse..st.j2ee.*", chuck);
    plugin2complead.put("org.eclipse..st.ejb.*", chuck);
    plugin2complead.put("org.eclipse..st.jsp.*", david);
    plugin2complead.put("org.eclipse..st.server.*", tim);
    plugin2complead.put("org.eclipse..st.servlet.*", chuck);
    plugin2complead.put("org.eclipse..st.ws", chris);
    plugin2complead.put("org.eclipse..st.ws[.].*", chris);
    plugin2complead.put("org.eclipse..st.command.*", chris);
    plugin2complead.put("org.eclipse..st.css.*", david);
    plugin2complead.put("org.eclipse..st.dtd.*", david);
    plugin2complead.put("org.eclipse..st.html.*", david);
    plugin2complead.put("org.eclipse..st.javascript.*", david);
    plugin2complead.put("org.eclipse..st.rdb.*", derping);
    plugin2complead.put("org.eclipse..st.sse.*", david);
    plugin2complead.put("org.eclipse..st.validation.*", chuck);
    plugin2complead.put("org.eclipse..st.web.*", chuck);
    plugin2complead.put("org.eclipse..st.wsdl.*", craig);
    plugin2complead.put("org.eclipse..st.xml.*", david);
    plugin2complead.put("org.eclipse..st.xsd.*", craig);
    plugin2complead.put("org.eclipse..st.internet.*", tim);
  }

  public Collection getFilters()
  {
    return filters;
  }

  public void setFilters(Collection filters)
  {
    this.filters = filters;
  }

  public String getOutput()
  {
    return output;
  }

  public void setOutput(String output)
  {
    this.output = output;
  }

  public Collection getExcludes()
  {
    return excludes;
  }

  public void setExcludes(Collection excludes)
  {
    this.excludes = excludes;
  }

  public Collection getIncludes()
  {
    return includes;
  }

  public void setIncludes(Collection includes)
  {
    this.includes = includes;
  }

  private boolean include(String s)
  {
    if (excludes != null && !excludes.isEmpty())
      for (Iterator it = excludes.iterator(); it.hasNext();)
        if (s.matches((String)it.next()))
          return false;
    if (includes != null && !includes.isEmpty())
    {
      for (Iterator it = includes.iterator(); it.hasNext();)
        if (s.matches((String)it.next()))
          return true;
      return false;
    }
    return true;
  }

  public void execute()
  {
    IExtensionPoint[] extpts = Platform.getExtensionRegistry().getExtensionPoints();
    for (int i = 0; i < extpts.length; i++)
    {
      String id = extpts[i].getUniqueIdentifier();
      String ns = extpts[i].getNamespace();
      if (id != null)
      {
        for (Iterator it = filters.iterator(); it.hasNext();)
        {
          if (id.matches((String)it.next()))
          {
            int extptRefCount = 0;
            IExtension[] exts = extpts[i].getExtensions();
            for (int j = 0; j < exts.length; j++)
            {
              if (include(exts[j].getNamespace()))
              {
                extptRefCount++;
              }
            }
            ComponentLead compLead = unknown;
            for (Iterator it3 = plugin2complead.keySet().iterator(); it3.hasNext();)
            {
              String regex = (String)it3.next();
              if (ns.matches(regex))
              {
                compLead = (ComponentLead)plugin2complead.get(regex);
                break;
              }
            }
            compLead.extpt2RefCount.put(id, String.valueOf(extptRefCount));
            break;
          }
        }
      }
    }
    genHTML();
  }

  private void genHTML()
  {
    try
    {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      baos.write("<root>".getBytes());
      writeCompLead(chuck, baos);
      writeCompLead(david, baos);
      writeCompLead(derping, baos);
      writeCompLead(chris, baos);
      writeCompLead(craig, baos);
      writeCompLead(tim, baos);
      writeCompLead(unknown, baos);
      baos.write("</root>".getBytes());
      baos.close();
      XSLUtil.transform
      (
        Platform.getBundle("org.eclipse.wtp.releng.tools.component.core").getResource("org/eclipse/wtp/releng/tools/component/xsl/extpt.xsl").openStream(),
        new ByteArrayInputStream(baos.toByteArray()),
        new FileOutputStream(output + "/extpt.html")
      );
    }
    catch (Throwable t)
    {
      t.printStackTrace();
      throw new RuntimeException(t);
    }
  }

  private void writeCompLead(ComponentLead compLead, ByteArrayOutputStream baos) throws IOException
  {
    baos.write("<team lead=\"".getBytes());
    baos.write(compLead.leadName.getBytes());
    baos.write("\">".getBytes());
    for (Iterator it = compLead.extpt2RefCount.keySet().iterator(); it.hasNext();)
    {
      String id = (String)it.next();
      String refCount = (String)compLead.extpt2RefCount.get(id);
      baos.write("<extpt id=\"".getBytes());
      baos.write(id.getBytes());
      baos.write("\" ref=\"".getBytes());
      baos.write(refCount.getBytes());
      baos.write("\"/>".getBytes());
    }
    baos.write("</team>".getBytes());
  }

  public Object run(Object arguments)
  {
    String extpt = System.getProperty("extpt");
    String output = System.getProperty("output");
    String includes = System.getProperty("includes");
    String excludes = System.getProperty("excludes");
    List args = new ArrayList();
    args.add("-extpt");
    args.addAll(tokenize(extpt));
    args.add("-output");
    args.add(output);
    if (includes != null)
    {
      args.add("-includes");
      args.addAll(tokenize(includes));
    }
    if (excludes != null)
    {
      args.add("-excludes");
      args.addAll(tokenize(excludes));
    }
    try
    {
      main((String[])args.toArray(new String[0]));
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
    return IPlatformRunnable.EXIT_OK;
  }

  private List tokenize(String s)
  {
    StringTokenizer st = new StringTokenizer(s, ",");
    List tokens = new ArrayList(st.countTokens());
    while(st.hasMoreTokens())
      tokens.add(st.nextToken());
    return tokens;
  }

  public static void main(String[] args)
  {
    CommandOptionParser optionParser = new CommandOptionParser(args);
    Map options = optionParser.getOptions();
    Collection extpt = (Collection)options.get("extpt");
    Collection output = (Collection)options.get("output");
    Collection includes = (Collection)options.get("includes");
    Collection excludes = (Collection)options.get("excludes");
    if (extpt == null || output == null || extpt.isEmpty() || output.isEmpty())
    {
      printUsage();
      System.exit(-1);
    }
    ExtensionPointScanner scanner = new ExtensionPointScanner();
    scanner.setFilters(extpt);
    scanner.setOutput((String)output.iterator().next());
    scanner.setIncludes(includes);
    scanner.setExcludes(excludes);
    scanner.execute();
  }

  private static void printUsage()
  {
    System.out.println("Usage: java org.eclipse.wtp.releng.tools.component.adopters.ExtensionPointScanner -extpt -output [options]");
    System.out.println("");
    System.out.println("\t-extpt\t\t<extpt>\t\tregular expressions for filtering your extension points");
    System.out.println("\t-output\t\t<output>\t\toutput directory");
    System.out.println("");
    System.out.println("where options include:");
    System.out.println("");
    System.out.println("\t-includes\t<includes>\tspace seperated extensions to include");
    System.out.println("\t-excludes\t<excludes>\tspace seperated extensions to exclude");
  }
}