package org.eclipse.wtp.releng.tools.bugs;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

public class UnverifiedBugsReminder
{
  private static final String DOWNLOAD_URL = "http://download.eclipse.org/webtools/downloads/";
  private static final String QUERY_UNVERIFIED_BUGS = "https://bugs.eclipse.org/bugs/report.cgi?bug_status=RESOLVED&resolution=FIXED&resolution=INVALID&resolution=WONTFIX&resolution=WORKSFORME&resolution=---&classification=WebTools&field0-0-0=days_elapsed&product=Web%20Tools&type0-0-0=greaterthan&value0-0-0=14&x_axis_field=bug_severity&y_axis_field=reporter&action=wrap&ctype=csv&format=table";
  private static final String QUERY_UNVERIFIED_BUGS_FOR_REPORTER = "https://bugs.eclipse.org/bugs/buglist.cgi?classification=WebTools&amp;product=Web+Tools&amp;bug_status=RESOLVED&amp;resolution=FIXED&amp;resolution=INVALID&amp;resolution=WONTFIX&amp;resolution=WORKSFORME&amp;resolution=---&amp;field0-0-0=days_elapsed&amp;type0-0-0=greaterthan&amp;value0-0-0=14&amp;emailreporter1=1&amp;emailtype1=substring&amp;email1=";

  private static final String SUBJECT = "[wtp bug] Reminder - Please verify resolved bugs";
  private static final String FROM = "jeffliu@ca.ibm.com";
  private static final String REPLY_TO = "wtp-dev@eclipse.org";
  private static final String MESSAGE = "Thank you for reporting the bugs that you found in WTP. One or more bugs have been fixed for more than two weeks [1]. Please verify the fixes using the latest 1.5.1 build [2]. Reporting and verifying bug fixes is an important way to improve WTP and we appreciate your contribution.";

  public static void main(String[] args)
  {
    if (args.length < 2)
    {
      System.out.println("Usage: java org.eclipse.wtp.releng.tools.bugs.UnverifiedBugsReminder <bugzilla id> <bugzilla password>");
      System.exit(0);
    }
    try
    {
      disableCertValidation();
      StringBuffer sb = new StringBuffer(QUERY_UNVERIFIED_BUGS);
      sb.append("&Bugzilla_login=");
      sb.append(args[0]);
      sb.append("&Bugzilla_password=");
      sb.append(args[1]);
      URL url = new URL(sb.toString());
      URLConnection conn = url.openConnection();
      if (conn.getContentType().indexOf("text/html") != -1)
      {
        System.out.println("Incorrect bugzilla id and password.");
        System.exit(0);
      }
      InputStream is = conn.getInputStream();
      BufferedWriter bw = new BufferedWriter(new FileWriter("unresolvedbugs.xml"));
      startScript(bw);
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String s = br.readLine(); // ignore the 1st line
      s = br.readLine(); 
      while (s != null)
      {
        int i = s.lastIndexOf("\"");
        if (i > 1)
          mailTask(bw, s.substring(1, i));
        s = br.readLine();
      }
      endScript(bw);
      bw.close();
    }
    catch (NoSuchAlgorithmException e)
    {
      e.printStackTrace();
    }
    catch (KeyManagementException e)
    {
      e.printStackTrace();
    }
    catch (MalformedURLException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private static void disableCertValidation() throws NoSuchAlgorithmException, KeyManagementException
  {
    SSLContext sc = SSLContext.getInstance("SSL");
    sc.init(null, new TrustManager[] {new MyX509TrustManager()}, new java.security.SecureRandom());
    HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
  }

  private static void startScript(BufferedWriter bw) throws IOException
  {
    bw.write("<project default=\"main\"><target name=\"main\">");
    bw.newLine();
    bw.newLine();
  }

  private static void endScript(BufferedWriter bw) throws IOException
  {
    bw.write("</target></project>");
  }

  private static String latestIBuildURL;

  private static void mailTask(BufferedWriter bw, String email) throws IOException
  {
    bw.write("<mail mailhost=\"na.relay.ibm.com\" subject=\"");
    bw.write(SUBJECT);
    bw.write("\">");
    bw.newLine();
    bw.write("<from address=\"");
    bw.write(FROM);
    bw.write("\"/>");
    bw.newLine();
    bw.write("<to address=\"");
    bw.write(email);
    bw.write("\"/>");
    bw.newLine();
    bw.write("<replyto address=\"");
    bw.write(REPLY_TO);
    bw.write("\"/>");
    bw.newLine();
    bw.write("<message>");
    bw.newLine();
    bw.write(MESSAGE);
    bw.newLine();
    bw.newLine();
    bw.write("[1] ");
    bw.write(QUERY_UNVERIFIED_BUGS_FOR_REPORTER);
    bw.write(email);
    bw.newLine();
    bw.newLine();
    bw.write("[2] ");
    if (latestIBuildURL == null)
      latestIBuildURL = getLatestIBuildURL();
    if (latestIBuildURL != null)
      bw.write(latestIBuildURL);
    bw.newLine();
    bw.newLine();
    bw.write("</message>");
    bw.newLine();
    bw.write("</mail>");
    bw.newLine();
    bw.newLine();
  }

  private static String getLatestIBuildURL()
  {
    Build latestBuild = null;
    try
    {
      URL url = new URL(DOWNLOAD_URL);
      InputStream is = url.openConnection().getInputStream();
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String s = br.readLine();
      while (s != null)
      {
        String start = "href=\"drops/";
        String end = "/";
        int sIndex = s.indexOf(start);
        int eIndex = s.indexOf(end, sIndex + start.length());
        while (sIndex != -1 && eIndex != -1)
        {
          String buildLabel = s.substring(sIndex + start.length(), eIndex);
          if (buildLabel != null)
          {
            String[] buildFragments = buildLabel.split("-");
            if (buildFragments.length > 2 && buildFragments[0].length() == 1 && buildFragments[2].length() == 12)
            {
              Build build = new Build(buildFragments[0], buildFragments[1], buildFragments[2].substring(0, 8), buildFragments[2].substring(8, 12));
              if (latestBuild == null || !latestBuild.newer(build))
              {
                latestBuild = build;
              }
            }
          }
          sIndex = s.indexOf(start, eIndex + end.length());
          eIndex = s.indexOf(end, sIndex + start.length());
        }
        s = br.readLine();
      }
      br.close();
    }
    catch (MalformedURLException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    if (latestBuild != null)
      return DOWNLOAD_URL + "drops/" + latestBuild.toString();
    else
      return DOWNLOAD_URL;
  }

  private static class MyX509TrustManager implements X509TrustManager
  {
    public java.security.cert.X509Certificate[] getAcceptedIssuers()
    {
      return null;
    }

    public void checkClientTrusted(java.security.cert.X509Certificate[] certs, String authType)
    {
    }

    public void checkServerTrusted(java.security.cert.X509Certificate[] certs, String authType)
    {
    }
  }

  private static class Build
  {
    private String type;
    private String id;
    private String date;
    private String time;

    public Build(String type, String id, String date, String time)
    {
      this.type = type;
      this.id = id;
      this.date = date;
      this.time = time;
    }

    public String getDate()
    {
      return date;
    }

    public void setDate(String date)
    {
      this.date = date;
    }

    public String getId()
    {
      return id;
    }

    public void setId(String id)
    {
      this.id = id;
    }

    public String getTime()
    {
      return time;
    }

    public void setTime(String time)
    {
      this.time = time;
    }

    public String getType()
    {
      return type;
    }

    public void setType(String type)
    {
      this.type = type;
    }

    public boolean newer(Build build)
    {
      int thisDate = Integer.parseInt(date);
      int thatDate = Integer.parseInt(build.getDate());
      if (thisDate == thatDate)
      {
        int thisTime = Integer.parseInt(time);
        int thatTime = Integer.parseInt(build.getTime());
        if (thisTime == thatTime)
          return getBuildTypeWeight(type) > getBuildTypeWeight(build.getType());
        else
          return thisTime > thatTime;
      }
      else
        return thisDate > thatDate;
    }

    private static final String BUILD_TYPE_S = "S";
    private static final String BUILD_TYPE_I = "I";
    private static final String BUILD_TYPE_N = "N";
    private static final String BUILD_TYPE_M = "M";

    private int getBuildTypeWeight(String buildType)
    {
      if (BUILD_TYPE_S.equals(buildType))
        return 40;
      else if (BUILD_TYPE_I.equals(buildType))
        return 30;
      else if (BUILD_TYPE_N.equals(buildType))
        return 20;
      else if (BUILD_TYPE_M.equals(buildType))
        return 10;
      else
        return 0;
    }

    public String toString()
    {
      StringBuffer sb = new StringBuffer();
      sb.append(type);
      sb.append("-");
      sb.append(id);
      sb.append("-");
      sb.append(date);
      sb.append(time);
      sb.append("/");
      return sb.toString();
    }
  }
}